@@ -292,6 +292,18 @@ public:
         return getWidth() * getHeight();
     }
 
+    /** \brief
+     * Gets the perimeter of this envelope.
+     *
+     * @return the perimeter of the envelope
+     * @return 0.0 if the envelope is null
+     */
+    double
+    getPerimeter() const
+    {
+        return 2*getWidth() + 2*getHeight();
+    }
+
     /** \brief
      * Returns true if this Envelope covers a finite region
      */
@@ -376,6 +388,16 @@ public:
      */
     bool intersection(const Envelope& env, Envelope& result) const;
 
+    /** \brief
+     * Computes the intersection of two [Envelopes](@ref Envelope).
+     *
+     * @param env the envelope to intersect with
+     * @return     the envelope representing the intersection of
+     *             the envelopes (this will be the null envelope
+     *             if either argument is null, or they do not intersect)
+     */
+    Envelope intersection(const Envelope& env) const;
+
     /** \brief
      * Translates this envelope by given amounts in the X and Y direction.
      *
@@ -590,7 +612,7 @@ public:
     * Tests if the region defined by other
     * is disjoint from the region of this Envelope
     *
-    * @param other  the Envelope being checked for disjointness
+    * @param other  the Envelope being checked for disjointedness
     * @return true if the Envelopes are disjoint
     */
     bool disjoint(const Envelope& other) const
