@@ -18,8 +18,7 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_GEOM_GEOMETRY_H
-#define GEOS_GEOM_GEOMETRY_H
+#pragma once
 
 #ifndef USE_UNSTABLE_GEOS_CPP_API
 #ifndef _MSC_VER
@@ -32,11 +31,10 @@
 #endif
 
 #include <geos/export.h>
-#include <geos/inline.h>
 #include <geos/geom/Envelope.h>
 #include <geos/geom/Dimension.h> // for Dimension::DimensionType
 #include <geos/geom/GeometryComponentFilter.h> // for inheritance
-#include <geos/geom/IntersectionMatrix.h>
+#include <geos/geom/CoordinateSequence.h> // to materialize CoordinateSequence
 
 #include <algorithm>
 #include <string>
@@ -62,17 +60,17 @@ class GeometryFactory;
 class GeometryFilter;
 class PrecisionModel;
 class Point;
+class IntersectionMatrix;
 }
 namespace io { // geos.io
-class Unload;
 } // namespace geos.io
 }
 
-namespace geos {
+namespace geos { // geos
 namespace geom { // geos::geom
 
 /// Geometry types
-enum GeometryTypeId {
+enum GeometryTypeId : int {
     /// a point
     GEOS_POINT,
     /// a linestring
@@ -88,7 +86,12 @@ enum GeometryTypeId {
     /// a collection of polygons
     GEOS_MULTIPOLYGON,
     /// a collection of heterogeneus geometries
-    GEOS_GEOMETRYCOLLECTION
+    GEOS_GEOMETRYCOLLECTION,
+    GEOS_CIRCULARSTRING,
+    GEOS_COMPOUNDCURVE,
+    GEOS_CURVEPOLYGON,
+    GEOS_MULTICURVE,
+    GEOS_MULTISURFACE,
 };
 
 enum GeometrySortIndex {
@@ -99,7 +102,12 @@ enum GeometrySortIndex {
     SORTINDEX_MULTILINESTRING = 4,
     SORTINDEX_POLYGON = 5,
     SORTINDEX_MULTIPOLYGON = 6,
-    SORTINDEX_GEOMETRYCOLLECTION = 7
+    SORTINDEX_GEOMETRYCOLLECTION = 7,
+    SORTINDEX_CIRCULARSTRING = 8,
+    SORTINDEX_COMPOUNDCURVE = 9,
+    SORTINDEX_CURVEPOLYGON = 10,
+    SORTINDEX_MULTICURVE = 11,
+    SORTINDEX_MULTISURFACE = 12,
 };
 
 /**
@@ -261,8 +269,7 @@ public:
      *
      * @return the ID of the coordinate space in which the Geometry is defined.
      */
-    virtual int
-    getSRID() const
+    int getSRID() const
     {
         return SRID;
     }
@@ -283,7 +290,7 @@ public:
     const PrecisionModel* getPrecisionModel() const;
 
     /// Returns a vertex of this Geometry, or NULL if this is the empty geometry.
-    virtual const Coordinate* getCoordinate() const = 0; //Abstract
+    virtual const CoordinateXY* getCoordinate() const = 0; //Abstract
 
     /**
      * \brief
@@ -301,11 +308,19 @@ public:
     /// Return a string representation of this Geometry type
     virtual std::string getGeometryType() const = 0; //Abstract
 
+    /// Returns whether the Geometry contains curved components
+    virtual bool hasCurvedComponents() const;
+
     /// Return an integer representation of this Geometry type
     virtual GeometryTypeId getGeometryTypeId() const = 0; //Abstract
 
-    /// Returns the number of geometries in this collection
-    /// (or 1 if this is not a collection)
+    /**
+     * \brief Returns the number of geometries in this collection,
+     * or 1 if this is not a collection.
+     *
+     * Empty collection or multi-geometry types return 0,
+     * and empty simple geometry types return 1.
+     */
     virtual std::size_t
     getNumGeometries() const
     {
@@ -329,7 +344,7 @@ public:
      *
      * @see IsValidOp
      */
-    virtual bool isValid() const;
+    bool isValid() const;
 
     /// Returns whether or not the set of points in this Geometry is empty.
     virtual bool isEmpty() const = 0; //Abstract
@@ -344,6 +359,11 @@ public:
     /// Returns the dimension of this Geometry (0=point, 1=line, 2=surface)
     virtual Dimension::DimensionType getDimension() const = 0; //Abstract
 
+    /// Checks whether any component of this geometry has dimension d
+    virtual bool hasDimension(Dimension::DimensionType d) const {
+        return getDimension() == d;
+    }
+
     /// Checks whether this Geometry consists only of components having dimension d.
     virtual bool isDimensionStrict(Dimension::DimensionType d) const {
         return d == getDimension();
@@ -361,6 +381,9 @@ public:
         return isDimensionStrict(Dimension::A);
     }
 
+    bool isMixedDimension() const;
+    bool isMixedDimension(Dimension::DimensionType* baseDim) const;
+
     bool isCollection() const {
         int t = getGeometryTypeId();
         return t == GEOS_GEOMETRYCOLLECTION ||
@@ -369,9 +392,22 @@ public:
                t == GEOS_MULTIPOLYGON;
     }
 
-    /// Returns the coordinate dimension of this Geometry (2=XY, 3=XYZ, 4=XYZM in future).
+    static GeometryTypeId multiTypeId(GeometryTypeId typeId) {
+        switch (typeId) {
+            case GEOS_POINT: return GEOS_MULTIPOINT;
+            case GEOS_LINESTRING: return GEOS_MULTILINESTRING;
+            case GEOS_POLYGON: return GEOS_MULTIPOLYGON;
+            default: return typeId;
+        }
+    }
+
+    /// Returns the coordinate dimension of this Geometry (2=XY, 3=XYZ or XYM, 4=XYZM).
     virtual uint8_t getCoordinateDimension() const = 0; //Abstract
 
+    virtual bool hasZ() const = 0;
+
+    virtual bool hasM() const = 0;
+
     /**
      * \brief
      * Returns the boundary, or an empty geometry of appropriate
@@ -400,7 +436,7 @@ public:
      * Returns the minimum and maximum x and y values in this Geometry,
      * or a null Envelope if this Geometry is empty.
      */
-    virtual const Envelope* getEnvelopeInternal() const;
+    virtual const Envelope* getEnvelopeInternal() const = 0;
 
     /**
      * Tests whether this geometry is disjoint from the specified geometry.
@@ -495,17 +531,14 @@ public:
     /// Returns the DE-9IM intersection matrix for the two Geometrys.
     std::unique_ptr<IntersectionMatrix> relate(const Geometry* g) const;
 
-    std::unique_ptr<IntersectionMatrix> relate(const Geometry& g) const
-    {
-        return relate(&g);
-    }
+    std::unique_ptr<IntersectionMatrix> relate(const Geometry& g) const;
 
     /**
      * \brief
      * Returns true if the DE-9IM intersection matrix for the two
      * Geometrys is T*F**FFF*.
      */
-    virtual bool equals(const Geometry* g) const;
+    bool equals(const Geometry* g) const;
 
     /** \brief
      * Returns <code>true</code> if this geometry covers the
@@ -577,11 +610,7 @@ public:
      * @see Geometry#within
      * @see Geometry#covers
      */
-    bool
-    coveredBy(const Geometry* g) const
-    {
-        return g->covers(this);
-    }
+    bool coveredBy(const Geometry* g) const;
 
 
     /// Returns the Well-known Text representation of this Geometry.
@@ -689,7 +718,7 @@ public:
      *   and dissolving the linework.
      * - Unioning a set of [Polygons](@ref Polygon) will always
      *   return a polygonal geometry (unlike Geometry::Union(const Geometry* other) const),
-     *   which may return geometrys of lower dimension if a topology collapse
+     *   which may return geometries of lower dimension if a topology collapse
      *   occurred.
      *
      * @return the union geometry
@@ -724,11 +753,19 @@ public:
 
     /** \brief
      * Returns true iff the two Geometrys are of the same type and their
-     * vertices corresponding by index are equal up to a specified tolerance.
+     * vertices corresponding by index are equal up to a specified distance
+     * tolerance. Geometries are not required to have the same dimemsion;
+     * any Z/M values are ignored.
      */
     virtual bool equalsExact(const Geometry* other, double tolerance = 0)
         const = 0; // Abstract
 
+    /** \brief
+     * Returns true if the two geometries are of the same type and their
+     * vertices corresponding by index are equal in all dimensions.
+     */
+    virtual bool equalsIdentical(const Geometry* other) const = 0;
+
     virtual void apply_rw(const CoordinateFilter* filter) = 0; //Abstract
     virtual void apply_ro(CoordinateFilter* filter) const = 0; //Abstract
     virtual void apply_rw(GeometryFilter* filter);
@@ -780,13 +817,7 @@ public:
     virtual void normalize() = 0; //Abstract
 
     /// Comparator for sorting geometry
-    virtual int compareTo(const Geometry* geom) const;
-
-    /** \brief
-     * Returns the minimum distance between this Geometry
-     * and the Geometry g
-     */
-    virtual double distance(const Geometry* g) const;
+    int compareTo(const Geometry* geom) const;
 
     /// Returns the area of this Geometry.
     virtual double getArea() const;
@@ -794,6 +825,14 @@ public:
     /// Returns the length of this Geometry.
     virtual double getLength() const;
 
+    /** Returns the minimum distance between this Geometry and the Geometry g
+     *
+     * @param g the Geometry to calculate distance to
+     * @return the distance in Cartesian units
+     */
+    virtual double distance(const Geometry* g) const;
+
+
     /** \brief
      * Tests whether the distance from this Geometry  to another
      * is less than or equal to a specified value.
@@ -823,7 +862,7 @@ public:
     //
     /// Returns false if centroid cannot be computed (EMPTY geometry)
     ///
-    virtual bool getCentroid(Coordinate& ret) const;
+    virtual bool getCentroid(CoordinateXY& ret) const;
 
     /** \brief
      * Computes an interior point of this <code>Geometry</code>.
@@ -842,20 +881,16 @@ public:
      * Notifies this Geometry that its Coordinates have been changed
      * by an external party (using a CoordinateFilter, for example).
      */
-    virtual void geometryChanged();
+    void geometryChanged();
 
     /**
      * \brief
      * Notifies this Geometry that its Coordinates have been changed
      * by an external party.
      */
-    void geometryChangedAction();
+    virtual void geometryChangedAction() = 0;
 
 protected:
-
-    /// The bounding box of this Geometry
-    mutable std::unique_ptr<Envelope> envelope;
-
     /// Make a deep-copy of this Geometry
     virtual Geometry* cloneImpl() const = 0;
 
@@ -889,23 +924,39 @@ protected:
     virtual bool isEquivalentClass(const Geometry* other) const;
 
     static void checkNotGeometryCollection(const Geometry* g);
-    // throw(IllegalArgumentException *);
-
-    //virtual void checkEqualSRID(Geometry *other);
-
-    //virtual void checkEqualPrecisionModel(Geometry *other);
-
-    virtual Envelope::Ptr computeEnvelopeInternal() const = 0; //Abstract
 
     virtual int compareToSameClass(const Geometry* geom) const = 0; //Abstract
 
-    int compare(std::vector<Coordinate> a, std::vector<Coordinate> b) const;
+    template<typename T>
+    static int compare(const T& a, const T& b)
+    {
+        std::size_t i = 0;
+        std::size_t j = 0;
+        while(i < a.size() && j < b.size()) {
+            const auto& aGeom = *a[i];
+            const auto& bGeom = *b[j];
 
-    int compare(std::vector<Geometry*> a, std::vector<Geometry*> b) const;
+            int comparison = aGeom.compareTo(&bGeom);
+            if(comparison != 0) {
+                return comparison;
+            }
 
-    int compare(const std::vector<std::unique_ptr<Geometry>> & a, const std::vector<std::unique_ptr<Geometry>> & b) const;
+            i++;
+            j++;
+        }
 
-    bool equal(const Coordinate& a, const Coordinate& b,
+        if(i < a.size()) {
+            return 1;
+        }
+
+        if(j < b.size()) {
+            return -1;
+        }
+
+        return 0;
+    }
+
+    bool equal(const CoordinateXY& a, const CoordinateXY& b,
                double tolerance) const;
     int SRID;
 
@@ -932,6 +983,10 @@ protected:
         return gv;
     }
 
+    static std::vector<std::unique_ptr<Geometry>> toGeometryArray(std::vector<std::unique_ptr<Geometry>> && v) {
+        return std::move(v);
+    }
+
 protected:
 
     virtual int getSortIndex() const = 0;
@@ -991,5 +1046,3 @@ struct GeomPtrPair {
 #ifdef _MSC_VER
 #pragma warning(pop)
 #endif
-
-#endif // ndef GEOS_GEOM_GEOMETRY_H
