@@ -16,25 +16,16 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_NODING_SEGMENTNODE_H
-#define GEOS_NODING_SEGMENTNODE_H
+#pragma once
 
 #include <geos/export.h>
+#include <geos/geom/Coordinate.h>
+#include <geos/noding/SegmentPointComparator.h>
+#include <geos/noding/SegmentString.h>
 
 #include <vector>
 #include <iostream>
 
-#include <geos/inline.h>
-
-#include <geos/geom/Coordinate.h>
-
-// Forward declarations
-namespace geos {
-namespace noding {
-class NodedSegmentString;
-}
-}
-
 namespace geos {
 namespace noding { // geos.noding
 
@@ -46,8 +37,6 @@ namespace noding { // geos.noding
  */
 class GEOS_DLL SegmentNode {
 private:
-    // const NodedSegmentString* segString;
-
     int segmentOctant;
 
     bool isInteriorVar;
@@ -56,7 +45,7 @@ public:
     friend std::ostream& operator<< (std::ostream& os, const SegmentNode& n);
 
     /// the point of intersection (own copy)
-    geom::Coordinate coord;
+    geom::CoordinateXYZM coord;
 
     /// the index of the containing line segment in the parent edge
     std::size_t segmentIndex;
@@ -73,9 +62,18 @@ public:
     ///
     /// @param nSegmentOctant
     ///
-    SegmentNode(const NodedSegmentString& ss,
-                const geom::Coordinate& nCoord,
-                std::size_t nSegmentIndex, int nSegmentOctant);
+    template<typename CoordType>
+    SegmentNode(const SegmentString& ss,
+        const CoordType& nCoord,
+        std::size_t nSegmentIndex, int nSegmentOctant)
+        : segmentOctant(nSegmentOctant)
+        , coord(nCoord)
+        , segmentIndex(nSegmentIndex)
+    {
+        // Number of points in NodedSegmentString is one-more number of segments
+        assert(segmentIndex < ss.size());
+        isInteriorVar = !coord.equals2D(ss.getCoordinate(segmentIndex));
+    }
 
     ~SegmentNode() {}
 
@@ -90,7 +88,16 @@ public:
         return isInteriorVar;
     }
 
-    bool isEndPoint(unsigned int maxSegmentIndex) const;
+    bool isEndPoint(unsigned int maxSegmentIndex) const
+    {
+        if(segmentIndex == 0 && ! isInteriorVar) {
+            return true;
+        }
+        if(segmentIndex == maxSegmentIndex) {
+            return true;
+        }
+        return false;
+    };
 
     /**
      * @return -1 this EdgeIntersection is located before
@@ -99,14 +106,37 @@ public:
      * @return 1 this EdgeIntersection is located after the
      *           argument location
      */
-    int compareTo(const SegmentNode& other) const;
+    int compareTo(const SegmentNode& other) const
+    {
+        if (segmentIndex < other.segmentIndex) {
+            return -1;
+        }
+        if (segmentIndex > other.segmentIndex) {
+            return 1;
+        }
+
+        if (coord.equals2D(other.coord)) {
+
+            return 0;
+        }
+
+        // an exterior node is the segment start point,
+        // so always sorts first
+        // this guards against a robustness problem
+        // where the octants are not reliable
+        if (!isInteriorVar) return -1;
+        if (!other.isInteriorVar) return 1;
+
+        return SegmentPointComparator::compare(
+            segmentOctant, coord,
+            other.coord);
+    };
 
-    //string print() const;
 };
 
-std::ostream& operator<< (std::ostream& os, const SegmentNode& n);
+// std::ostream& operator<< (std::ostream& os, const SegmentNode& n);
 
-struct GEOS_DLL  SegmentNodeLT {
+struct GEOS_DLL SegmentNodeLT {
     bool
     operator()(SegmentNode* s1, SegmentNode* s2) const
     {
@@ -124,8 +154,8 @@ struct GEOS_DLL  SegmentNodeLT {
 } // namespace geos.noding
 } // namespace geos
 
-#ifdef GEOS_INLINE
-# include "geos/noding/SegmentNode.inl"
-#endif
 
-#endif // GEOS_NODING_SEGMENTNODE_H
+
+
+
+
