@@ -17,8 +17,7 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_OP_PREDICATE_SEGMENTINTERSECTIONTESTER_H
-#define GEOS_OP_PREDICATE_SEGMENTINTERSECTIONTESTER_H
+#pragma once
 
 #include <geos/export.h>
 
@@ -90,5 +89,3 @@ public:
 } // namespace geos::operation::predicate
 } // namespace geos::operation
 } // namespace geos
-
-#endif // ifndef GEOS_OP_PREDICATE_SEGMENTINTERSECTIONTESTER_H
