@@ -1,14 +1,13 @@
 /**********************************************************************
- * $Id: RelateNodeFactory.h 2557 2009-06-08 09:30:55Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2006 Refractions Research Inc.
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU Lesser General Public Licence as published
- * by the Free Software Foundation. 
+ * by the Free Software Foundation.
  * See the COPYING file for more information.
  *
  **********************************************************************
@@ -17,8 +16,7 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_OP_RELATE_RELATENODEFACTORY_H
-#define GEOS_OP_RELATE_RELATENODEFACTORY_H
+#pragma once
 
 #include <geos/export.h>
 
@@ -26,12 +24,12 @@
 
 // Forward declarations
 namespace geos {
-	namespace geom {
-		class Coordinate;
-	}
-	namespace geomgraph {
-		class Node;
-	}
+namespace geom {
+class Coordinate;
+}
+namespace geomgraph {
+class Node;
+}
 }
 
 
@@ -44,10 +42,10 @@ namespace relate { // geos::operation::relate
  */
 class GEOS_DLL RelateNodeFactory: public geomgraph::NodeFactory {
 public:
-	geomgraph::Node* createNode(const geom::Coordinate &coord) const;
-	static const geomgraph::NodeFactory &instance();
+    geomgraph::Node* createNode(const geom::Coordinate& coord) const override;
+    static const geomgraph::NodeFactory& instance();
 private:
-	RelateNodeFactory() {};
+    RelateNodeFactory() {}
 };
 
 
@@ -55,12 +53,3 @@ private:
 } // namespace geos:operation
 } // namespace geos
 
-#endif // GEOS_OP_RELATE_RELATENODEFACTORY_H
-
-/**********************************************************************
- * $Log$
- * Revision 1.1  2006/03/21 13:11:29  strk
- * opRelate.h header split
- *
- **********************************************************************/
-
