@@ -16,8 +16,7 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_PRECISION_GEOMETRYPRECISIONREDUCER_H
-#define GEOS_PRECISION_GEOMETRYPRECISIONREDUCER_H
+#pragma once
 
 #include <geos/export.h>
 #include <geos/geom/GeometryFactory.h> // for GeometryFactory::Ptr
@@ -38,7 +37,22 @@ namespace precision { // geos.precision
 /** \brief
  * Reduces the precision of a {@link geom::Geometry}
  * according to the supplied {@link geom::PrecisionModel},
- * ensuring that the result is topologically valid.
+ * ensuring that the result is valid (unless specified otherwise).
+ *
+ * By default the reduced result is topologically valid
+ * To ensure this a polygonal geometry is reduced in a topologically valid fashion
+ * (technically, by using snap-rounding).
+ * It can be forced to be reduced pointwise by using setPointwise(boolean).
+ * Note that in this case the result geometry may be invalid.
+ * Linear and point geometry is always reduced pointwise (i.e. without further change to
+ * its topology or structure), since this does not change validity.
+ *
+ * By default the geometry precision model is not changed.
+ * This can be overridden by usingsetChangePrecisionModel(boolean).
+ *
+ * Normally collapsed components (e.g. lines collapsing to a point)
+ * are not included in the result.
+ * This behavior can be changed by using setRemoveCollapsedComponents(boolean).
  */
 class GEOS_DLL GeometryPrecisionReducer {
 
@@ -50,18 +64,28 @@ private:
     const geom::PrecisionModel& targetPM;
 
     bool removeCollapsed;
-
+    bool changePrecisionModel;
+    bool useAreaReducer;
     bool isPointwise;
 
-    std::unique_ptr<geom::Geometry> reducePointwise(const geom::Geometry& geom);
-
-    std::unique_ptr<geom::Geometry> fixPolygonalTopology(
-        const geom::Geometry& geom);
+    std::unique_ptr<geom::Geometry> fixPolygonalTopology(const geom::Geometry& geom);
 
     geom::GeometryFactory::Ptr createFactory(
         const geom::GeometryFactory& oldGF,
         const geom::PrecisionModel& newPM);
 
+    /**
+    * Duplicates a geometry to one that uses a different PrecisionModel,
+    * without changing any coordinate values.
+    *
+    * @param geom the geometry to duplicate
+    * @param newPM the precision model to use
+    * @return the geometry value with a new precision model
+    */
+    std::unique_ptr<geom::Geometry> changePM(
+        const geom::Geometry* geom,
+        const geom::PrecisionModel& newPM);
+
     GeometryPrecisionReducer(GeometryPrecisionReducer const&); /*= delete*/
     GeometryPrecisionReducer& operator=(GeometryPrecisionReducer const&); /*= delete*/
 
@@ -79,41 +103,21 @@ public:
      * @return the reduced geometry
      */
     static std::unique_ptr<geom::Geometry>
-    reduce(
-        const geom::Geometry& g,
-        const geom::PrecisionModel& precModel)
-    {
-        GeometryPrecisionReducer reducer(precModel);
-        return reducer.reduce(g);
-    }
+    reduce(const geom::Geometry& g, const geom::PrecisionModel& precModel);
 
-    /**
-     * Convenience method for doing precision reduction
-     * on a single geometry,
-     * with collapses removed
-     * and keeping the geometry precision model the same,
-     * but NOT preserving valid polygonal topology.
-     *
-     * @param g the geometry to reduce
-     * @param precModel the precision model to use
-     * @return the reduced geometry
-     */
     static std::unique_ptr<geom::Geometry>
-    reducePointwise(
-        const geom::Geometry& g,
-        const geom::PrecisionModel& precModel)
-    {
-        GeometryPrecisionReducer reducer(precModel);
-        reducer.setPointwise(true);
-        return reducer.reduce(g);
-    }
+    reducePointwise(const geom::Geometry& g, const geom::PrecisionModel& precModel);
+
+    static std::unique_ptr<geom::Geometry>
+    reduceKeepCollapsed(const geom::Geometry& g, const geom::PrecisionModel& precModel);
 
     GeometryPrecisionReducer(const geom::PrecisionModel& pm)
-        :
-        newFactory(nullptr),
-        targetPM(pm),
-        removeCollapsed(true),
-        isPointwise(false)
+        : newFactory(nullptr)
+        , targetPM(pm)
+        , removeCollapsed(true)
+        , changePrecisionModel(false)
+        , useAreaReducer(false)
+        , isPointwise(false)
     {}
 
     /**
@@ -121,19 +125,26 @@ public:
      * Create a reducer that will change the precision model of the
      * new reduced Geometry
      *
-     * @param gf the factory for the created Geometry.
+     * @param changeFactory the factory for the created Geometry.
      *           Its PrecisionModel will be used for the reduction.
      *           NOTE: ownership left to caller must be kept alive for
      *           the whole lifetime of the returned Geometry.
      */
-    GeometryPrecisionReducer(const geom::GeometryFactory& gf);
+    GeometryPrecisionReducer(const geom::GeometryFactory& changeFactory)
+        : newFactory(&changeFactory)
+        , targetPM(*(changeFactory.getPrecisionModel()))
+        , removeCollapsed(true)
+        , changePrecisionModel(false)
+        , useAreaReducer(false)
+        , isPointwise(false)
+    {}
 
     /**
      * Sets whether the reduction will result in collapsed components
      * being removed completely, or simply being collapsed to an (invalid)
      * Geometry of the same type.
      *
-     * @param remove if <code>true</code> collapsed components will be removed
+     * @param remove if true collapsed components will be removed
      */
     void
     setRemoveCollapsedComponents(bool remove)
@@ -141,6 +152,28 @@ public:
         removeCollapsed = remove;
     }
 
+    /** \brief
+    * Sets whether the {@link geom::PrecisionModel} of the new reduced Geometry
+    * will be changed to be the {@link geom::PrecisionModel} supplied to
+    * specify the precision reduction.
+    * The default is to not change the precision model
+    *
+    * @param change if true the precision
+    * model of the created Geometry will be the
+    * the precisionModel supplied in the constructor.
+    */
+    void
+    setChangePrecisionModel(bool change)
+    {
+        changePrecisionModel = change;
+    }
+
+    void
+    setUseAreaReducer(bool useAR)
+    {
+        useAreaReducer = useAR;
+    }
+
     /** \brief
      * Sets whether the precision reduction will be done
      * in pointwise fashion only.
@@ -160,9 +193,10 @@ public:
 
     std::unique_ptr<geom::Geometry> reduce(const geom::Geometry& geom);
 
+
+
 };
 
 } // namespace geos.precision
 } // namespace geos
 
-#endif // GEOS_PRECISION_GEOMETRYPRECISIONREDUCER_H
